#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"column.h"
#include	"fileio.h"

#define	BUFFER_SIZE		4096
#define	EOF_CHAR			0x1a

/* ---------------------------------------------------------------------
	Set up the input and output files with their buffers.
   --------------------------------------------------------------------- */

void	buffers(FILEIO* pFile, char *pszFilename, BOOL fRead, int iRecLen, BOOL fTextMode)
{
	pFile->pchBuffer = 0;
	pFile->pchRecord = 0;

	if ((pFile->hFile = fopen(pszFilename, fRead ? "rb" : "wb")) == 0)
		error_handler(1, pszFilename);

	if ((pFile->pchBuffer = (char*) malloc(BUFFER_SIZE)) == 0)
		error_handler(4, pszFilename);

	if (setvbuf(pFile->hFile, pFile->pchBuffer, _IOFBF, BUFFER_SIZE) != 0)
		error_handler(2, pszFilename);

	pFile->iRecLen = iRecLen;

	if ((pFile->pchRecord = (char*) malloc(iRecLen)) == 0)
		error_handler(4, pszFilename);

	pFile->fTextMode = fTextMode;
}

/* ---------------------------------------------------------------------
	Read a single record as fixed length or variable delimited with CR-LF.
	Return value is FALSE if end-of-file encountered.
   --------------------------------------------------------------------- */

BOOL	read_record(FILEIO *pFile)
{
	int	i;

	if (feof(pFile->hFile))
		return FALSE;

	if (pFile->fTextMode) {
		if (0 == fgets(pFile->pchRecord, pFile->iRecLen, pFile->hFile)) {
			if (feof(pFile->hFile))
				return FALSE;

			error_handler(10,"input file");
		}

		/* skip saving of CR & LF chars */
		i = strlen(pFile->pchRecord);
		if (i > 0 && pFile->pchRecord[i-1] == '\n') i--;
		if (i > 0 && pFile->pchRecord[i-1] == '\r') i--;

		memset(&pFile->pchRecord[i], 0, pFile->iRecLen - i);
	}
	else {
		i = fread(pFile->pchBuffer, sizeof(char), pFile->iRecLen, pFile->hFile);
		if (i != pFile->iRecLen) {
			if (i == 0 && feof(pFile->hFile))
				return FALSE;

			if (i == 1 && feof(pFile->hFile) && pFile->pchBuffer[0] == EOF_CHAR)
				return FALSE;

			error_handler(10,"input file");
		}
	}
	return TRUE;
}

/* ---------------------------------------------------------------------
	Write a single record to a file.
	The arguments are:
		pFile			=	output file
		pchOutput	=	buffer containing record
		reclen		=	maximum / actual record length
	No return value.
   --------------------------------------------------------------------- */

void	write_record(FILEIO *pFile, const char *pchOutput, int iRecLen)
{
	int	i, cch;

	if (pFile->fTextMode) {
		for (i = 0, cch = 0; (i < iRecLen-1) && (pchOutput[i] != 0); i++)
			cch++;

		if (cch > 0 && fwrite(pchOutput, cch, 1, pFile->hFile) != 1)
			error_handler(10,"output file");

		fputs("\r\n", pFile->hFile);
	}
	else {
		if (fwrite(pchOutput, iRecLen, 1, pFile->hFile) != 1)
			error_handler(10,"output file");
	}
}

/* ---------------------------------------------------------------------
	Free any dynamically allocated storage and close all open files.
   --------------------------------------------------------------------- */

void	FileIOCleanup(FILEIO* pFile)
{
	if (pFile->hFile != 0)
		if (fclose(pFile->hFile) != 0)
			error_handler(3, 0);

	free(pFile->pchBuffer);
	free(pFile->pchRecord);
}


